@@ -1,24 +1,15 @@
 //
-// "$Id: CodeEditor.h 5498 2006-09-29 21:02:27Z mike $"
+// "$Id: CodeEditor.h 9980 2013-09-21 16:41:23Z greg.ercolano $"
 //
 // Code editor widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -47,9 +38,6 @@ class CodeEditor : public Fl_Text_Editor {
   static const char * const code_types[];
 
 
-  // 'compare_keywords()' - Compare two keywords...
-  static int compare_keywords(const void *a, const void *b);
-
   // 'style_parse()' - Parse text and produce style data.
   static void style_parse(const char *text, char *style, int length);
 
@@ -68,6 +56,10 @@ class CodeEditor : public Fl_Text_Editor {
   CodeEditor(int X, int Y, int W, int H, const char *L=0);
   ~CodeEditor();
   int top_line() { return get_absolute_top_line_number(); }
+
+  // attempt to make the fluid code editor widget honour textsize setting
+  void textsize(Fl_Fontsize s);
+  
 };
 
 class CodeViewer : public CodeEditor {
@@ -75,7 +67,7 @@ class CodeViewer : public CodeEditor {
   public:
 
   CodeViewer(int X, int Y, int W, int H, const char *L=0);
-  
+
   protected:
 
   int handle(int ev) { return Fl_Text_Display::handle(ev); }
@@ -84,5 +76,5 @@ class CodeViewer : public CodeEditor {
 #endif // !CodeEditor_h
 
 //
-// End of "$Id: CodeEditor.h 5498 2006-09-29 21:02:27Z mike $".
+// End of "$Id: CodeEditor.h 9980 2013-09-21 16:41:23Z greg.ercolano $".
 //
