@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Window_Type.cxx 8089 2010-12-20 22:19:24Z matt $"
+// "$Id: Fl_Window_Type.cxx 9080 2011-09-28 20:31:27Z matt $"
 //
 // Window type code for the Fast Light Tool Kit (FLTK).
 //
@@ -9,20 +9,11 @@
 //
 // Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -1485,6 +1476,17 @@ void Fl_Widget_Class_Type::read_property(const char *c) {
   }
 }
 
+// Convert A::B::C::D to D (i.e. keep only innermost name)
+// This is useful for classes that contain a namespace component
+static const char *trimclassname(const char *n) {
+  const char *nn;
+  while((nn = strstr(n, "::"))) {
+    n = nn + 2;
+  }
+  return(n);
+}
+
+
 void Fl_Widget_Class_Type::write_code1() {
 #if 0
   Fl_Widget_Type::write_code1();
@@ -1498,43 +1500,43 @@ void Fl_Widget_Class_Type::write_code1() {
 
   write_h("\nclass %s : public %s {\n", name(), c);
   if (strstr(c, "Window")) {
-    write_h("  void _%s();\n", name());
+    write_h("  void _%s();\n", trimclassname(name()));
     write_h("public:\n");
-    write_h("  %s(int X, int Y, int W, int H, const char *L = 0);\n", name());
-    write_h("  %s(int W, int H, const char *L = 0);\n", name());
-    write_h("  %s();\n", name());
+    write_h("  %s(int X, int Y, int W, int H, const char *L = 0);\n", trimclassname(name()));
+    write_h("  %s(int W, int H, const char *L = 0);\n", trimclassname(name()));
+    write_h("  %s();\n", trimclassname(name()));
 
     // a constructor with all four dimensions plus label
-    write_c("%s::%s(int X, int Y, int W, int H, const char *L)\n", name(), name());
+    write_c("%s::%s(int X, int Y, int W, int H, const char *L)\n", name(), trimclassname(name()));
     write_c("  : %s(X, Y, W, H, L) {\n", c);
-    write_c("  _%s();\n", name());
+    write_c("  _%s();\n", trimclassname(name()));
     write_c("}\n\n");
 
     // a constructor with just the size and label. The window manager will position the window
-    write_c("%s::%s(int W, int H, const char *L)\n", name(), name());
+    write_c("%s::%s(int W, int H, const char *L)\n", name(), trimclassname(name()));
     write_c("  : %s(0, 0, W, H, L) {\n", c);
     write_c("  clear_flag(16);\n");
-    write_c("  _%s();\n", name());
+    write_c("  _%s();\n", trimclassname(name()));
     write_c("}\n\n");
     
     // a constructor that takes size and label from the Fluid database
-    write_c("%s::%s()\n", name(), name());
+    write_c("%s::%s()\n", name(), trimclassname(name()));
     write_c("  : %s(0, 0, %d, %d, ", c, o->w(), o->h());
     const char *cstr = label();
     if (cstr) write_cstring(cstr);
     else write_c("0");
     write_c(") {\n");
     write_c("  clear_flag(16);\n");
-    write_c("  _%s();\n", name());
+    write_c("  _%s();\n", trimclassname(name()));
     write_c("}\n\n");
     
-    write_c("void %s::_%s() {\n", name(), name());
+    write_c("void %s::_%s() {\n", name(), trimclassname(name()));
 //    write_c("  %s *w = this;\n", name());
   } else {
     write_h("public:\n");
-    write_h("  %s(int X, int Y, int W, int H, const char *L = 0);\n", name());
+    write_h("  %s(int X, int Y, int W, int H, const char *L = 0);\n", trimclassname(name()));
 
-    write_c("%s::%s(int X, int Y, int W, int H, const char *L)\n", name(), name());
+    write_c("%s::%s(int X, int Y, int W, int H, const char *L)\n", name(), trimclassname(name()));
     if (wc_relative)
       write_c("  : %s(0, 0, W, H, L) {\n", c);
     else
@@ -1594,5 +1596,5 @@ void Fl_Window_Type::copy_properties() {
 
 
 //
-// End of "$Id: Fl_Window_Type.cxx 8089 2010-12-20 22:19:24Z matt $".
+// End of "$Id: Fl_Window_Type.cxx 9080 2011-09-28 20:31:27Z matt $".
 //
