@@ -1,24 +1,15 @@
 //
-// "$Id: align_widget.cxx 5845 2007-05-20 00:01:06Z mike $"
+// "$Id: align_widget.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // Alignment code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -521,7 +512,7 @@ void widget_size_cb(Fl_Widget *, long size) {
       }
 
       Fl_Widget *w = ((Fl_Widget_Type *)o)->o;
-      w->labelsize((uchar)size);
+      w->labelsize((Fl_Font)size);
       Fl_Font f;
       int s = (int)size;
       Fl_Color c;
@@ -536,6 +527,6 @@ void widget_size_cb(Fl_Widget *, long size) {
 
 
 //
-// End of "$Id: align_widget.cxx 5845 2007-05-20 00:01:06Z mike $".
+// End of "$Id: align_widget.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
 
