@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Button.cxx 6589 2008-12-17 10:47:09Z fabien $"
+// "$Id: Fl_Button.cxx 10386 2014-10-19 20:17:17Z AlbrechtS $"
 //
 // Button widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2014 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file. If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -30,10 +21,23 @@
 #include <FL/Fl_Group.H>
 #include <FL/Fl_Window.H>
 
+#include <FL/Fl_Radio_Button.H>
+#include <FL/Fl_Toggle_Button.H>
+
+
+Fl_Widget_Tracker *Fl_Button::key_release_tracker = 0;
+
+
 // There are a lot of subclasses, named Fl_*_Button.  Some of
 // them are implemented by setting the type() value and testing it
 // here.  This includes Fl_Radio_Button and Fl_Toggle_Button
 
+/**
+  Sets the current value of the button.
+  A non-zero value sets the button to 1 (ON), and zero sets it to 0 (OFF).
+  \param[in] v button value.
+  \see set(), clear()
+ */
 int Fl_Button::value(int v) {
   v = v ? 1 : 0;
   oldval = v;
@@ -48,9 +52,13 @@ int Fl_Button::value(int v) {
   }
 }
 
+/**
+  Turns on this button and turns off all other radio buttons in the group
+  (calling \c value(1) or \c set() does not do this).
+ */
 void Fl_Button::setonly() { // set this radio button on, turn others off
   value(1);
-  Fl_Group* g = (Fl_Group*)parent();
+  Fl_Group* g = parent();
   Fl_Widget*const* a = g->array();
   for (int i = g->children(); i--;) {
     Fl_Widget* o = *a++;
@@ -62,6 +70,7 @@ void Fl_Button::draw() {
   if (type() == FL_HIDDEN_BUTTON) return;
   Fl_Color col = value() ? selection_color() : color();
   draw_box(value() ? (down_box()?down_box():fl_down(box())) : box(), col);
+  draw_backdrop();
   if (labeltype() == FL_NORMAL_LABEL && value()) {
     Fl_Color c = labelcolor();
     labelcolor(fl_contrast(c, col));
@@ -74,18 +83,18 @@ void Fl_Button::draw() {
 int Fl_Button::handle(int event) {
   int newval;
   switch (event) {
-  case FL_ENTER:
+  case FL_ENTER: /* FALLTHROUGH */
   case FL_LEAVE:
 //  if ((value_?selection_color():color())==FL_GRAY) redraw();
     return 1;
   case FL_PUSH:
     if (Fl::visible_focus() && handle(FL_FOCUS)) Fl::focus(this);
+    /* FALLTHROUGH */
   case FL_DRAG:
     if (Fl::event_inside(this)) {
       if (type() == FL_RADIO_BUTTON) newval = 1;
       else newval = !oldval;
-    } else
-    {
+    } else {
       clear_changed();
       newval = oldval;
     }
@@ -107,28 +116,19 @@ int Fl_Button::handle(int event) {
     else {
       value(oldval);
       set_changed();
-      if (when() & FL_WHEN_CHANGED) do_callback();
+      if (when() & FL_WHEN_CHANGED) {
+	Fl_Widget_Tracker wp(this);
+        do_callback();
+        if (wp.deleted()) return 1;
+      }
     }
     if (when() & FL_WHEN_RELEASE) do_callback();
     return 1;
   case FL_SHORTCUT:
     if (!(shortcut() ?
 	  Fl::test_shortcut(shortcut()) : test_shortcut())) return 0;
-    
     if (Fl::visible_focus() && handle(FL_FOCUS)) Fl::focus(this);
-
-    if (type() == FL_RADIO_BUTTON && !value_) {
-      setonly();
-      set_changed();
-      if (when() & (FL_WHEN_CHANGED|FL_WHEN_RELEASE)) 
-	do_callback();
-    } else if (type() == FL_TOGGLE_BUTTON) {
-      value(!value());
-      set_changed();
-      if (when() & (FL_WHEN_CHANGED|FL_WHEN_RELEASE)) 
-	do_callback();
-    } else if (when() & FL_WHEN_RELEASE) do_callback();
-    return 1;
+    goto triggered_by_keyboard;
   case FL_FOCUS :
   case FL_UNFOCUS :
     if (Fl::visible_focus()) {
@@ -142,17 +142,25 @@ int Fl_Button::handle(int event) {
       } else redraw();
       return 1;
     } else return 0;
+    /* NOTREACHED */
   case FL_KEYBOARD :
     if (Fl::focus() == this && Fl::event_key() == ' ' &&
         !(Fl::event_state() & (FL_SHIFT | FL_CTRL | FL_ALT | FL_META))) {
       set_changed();
-      if (type() == FL_RADIO_BUTTON && !value_) {
-	setonly();
-	if (when() & FL_WHEN_CHANGED) do_callback();
+    triggered_by_keyboard:
+      Fl_Widget_Tracker wp(this);
+      if (type() == FL_RADIO_BUTTON) {
+        if (!value_) {
+	  setonly();
+	  if (when() & FL_WHEN_CHANGED) do_callback();
+        }
       } else if (type() == FL_TOGGLE_BUTTON) {
 	value(!value());
 	if (when() & FL_WHEN_CHANGED) do_callback();
+      } else {
+        simulate_key_action();
       }
+      if (wp.deleted()) return 1;
       if (when() & FL_WHEN_RELEASE) do_callback();
       return 1;
     }
@@ -161,8 +169,49 @@ int Fl_Button::handle(int event) {
   }
 }
 
-Fl_Button::Fl_Button(int X, int Y, int W, int H, const char *l)
-: Fl_Widget(X,Y,W,H,l) {
+void Fl_Button::simulate_key_action()
+{
+  if (key_release_tracker) {
+    Fl::remove_timeout(key_release_timeout, key_release_tracker);
+    key_release_timeout(key_release_tracker);
+  }
+  value(1);
+  redraw();
+  key_release_tracker = new Fl_Widget_Tracker(this);
+  Fl::add_timeout(0.15, key_release_timeout, key_release_tracker);
+}
+
+void Fl_Button::key_release_timeout(void *d)
+{
+  Fl_Widget_Tracker *wt = (Fl_Widget_Tracker*)d;
+  if (!wt)
+    return;
+  if (wt==key_release_tracker)
+    key_release_tracker = 0L;
+  Fl_Button *btn = (Fl_Button*)wt->widget();
+  if (btn) {
+    btn->value(0);
+    btn->redraw();
+  }
+  delete wt;
+}
+
+/**
+  The constructor creates the button using the given position, size, and label.
+
+  The default box type is box(FL_UP_BOX).
+
+  You can control how the button is drawn when ON by setting down_box().
+  The default is FL_NO_BOX (0) which will select an appropriate box type
+  using the normal (OFF) box type by using fl_down(box()).
+
+  Derived classes may handle this differently.
+
+  \param[in] X, Y, W, H position and size of the widget
+  \param[in] L widget label, default is no label
+ */
+Fl_Button::Fl_Button(int X, int Y, int W, int H, const char *L)
+: Fl_Widget(X,Y,W,H,L) {
   box(FL_UP_BOX);
   down_box(FL_NO_BOX);
   value_ = oldval = 0;
@@ -170,6 +219,34 @@ Fl_Button::Fl_Button(int X, int Y, int W, int H, const char *l)
   set_flag(SHORTCUT_LABEL);
 }
 
+/**
+  The constructor creates the button using the given position, size, and label.
+
+  The Button type() is set to FL_RADIO_BUTTON.
+
+  \param[in] X, Y, W, H position and size of the widget
+  \param[in] L widget label, default is no label
+ */
+Fl_Radio_Button::Fl_Radio_Button(int X,int Y,int W,int H,const char *L)
+: Fl_Button(X, Y, W, H, L) {
+  type(FL_RADIO_BUTTON);
+}
+
+/**
+  The constructor creates the button using the given position, size, and label.
+
+  The Button type() is set to FL_TOGGLE_BUTTON.
+
+  \param[in] X, Y, W, H position and size of the widget
+  \param[in] L widget label, default is no label
+ */
+Fl_Toggle_Button::Fl_Toggle_Button(int X,int Y,int W,int H,const char *L)
+: Fl_Button(X,Y,W,H,L)
+{
+  type(FL_TOGGLE_BUTTON);
+}
+
+
 //
-// End of "$Id: Fl_Button.cxx 6589 2008-12-17 10:47:09Z fabien $".
+// End of "$Id: Fl_Button.cxx 10386 2014-10-19 20:17:17Z AlbrechtS $".
 //
