@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Check_Browser.cxx 5985 2007-11-20 21:15:08Z mike $"
+// "$Id: Fl_Check_Browser.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // Fl_Check_Browser header file for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -85,7 +76,8 @@ int Fl_Check_Browser::lineno(cb_item *p0) const {
 }
 
 Fl_Check_Browser::Fl_Check_Browser(int X, int Y, int W, int H, const char *l)
-	: Fl_Browser_(X, Y, W, H, l) {
+/**  The constructor makes an empty browser.*/
+: Fl_Browser_(X, Y, W, H, l) {
 	type(FL_SELECT_BROWSER);
 	when(FL_WHEN_NEVER);
 	first = last = 0;
@@ -164,11 +156,19 @@ int Fl_Check_Browser::item_selected(void *v) const {
 	cb_item *i = (cb_item *)v;
 	return i->selected;
 }
-
+/**
+ Add a new unchecked line to the end of the browser.  
+ \see add(char *s, int b)
+*/
 int Fl_Check_Browser::add(char *s) {
 	return (add(s, 0));
 }
 
+/** 
+ Add a new line to the end of the browser.  The text is copied
+ using the strdup() function.  It may also be NULL to make
+ a blank line.  It can set the item checked if \p b is not 0.
+ */
 int Fl_Check_Browser::add(char *s, int b) {
 	cb_item *p = (cb_item *)malloc(sizeof(cb_item));
 	p->next = 0;
@@ -193,6 +193,10 @@ int Fl_Check_Browser::add(char *s, int b) {
 	return (nitems_);
 }
 
+/**
+  Remove line n and make the browser one line shorter. Returns the 
+  number of lines left in the browser.
+*/
 int Fl_Check_Browser::remove(int item) {
   cb_item *p = find_item(item);
   
@@ -225,6 +229,7 @@ int Fl_Check_Browser::remove(int item) {
   return (nitems_);
 }
 
+/**  Remove every item from the browser.*/
 void Fl_Check_Browser::clear() {
 	cb_item *p = first;
 	cb_item *next;
@@ -246,6 +251,7 @@ void Fl_Check_Browser::clear() {
 	cached_item = -1;
 }
 
+/** Gets the current status of item item. */
 int Fl_Check_Browser::checked(int i) const {
 	cb_item *p = find_item(i);
 
@@ -253,6 +259,7 @@ int Fl_Check_Browser::checked(int i) const {
 	return 0;
 }
 
+/** Sets the check status of item item to b. */
 void Fl_Check_Browser::checked(int i, int b) {
 	cb_item *p = find_item(i);
 
@@ -267,10 +274,12 @@ void Fl_Check_Browser::checked(int i, int b) {
 	}
 }
 
+/**  Returns the index of the currently selected item.*/
 int Fl_Check_Browser::value() const {
 	return lineno((cb_item *)selection());
 }
 
+/**  Return a pointer to an internal buffer holding item item's text.*/
 char *Fl_Check_Browser::text(int i) const {
 	cb_item *p = find_item(i);
 
@@ -278,6 +287,7 @@ char *Fl_Check_Browser::text(int i) const {
 	return 0;
 }
 
+/**  Sets all the items checked.*/
 void Fl_Check_Browser::check_all() {
 	cb_item *p;
 
@@ -288,6 +298,7 @@ void Fl_Check_Browser::check_all() {
 	redraw();
 }
 
+/**  Sets all the items unchecked.*/
 void Fl_Check_Browser::check_none() {
 	cb_item *p;
 
@@ -305,5 +316,5 @@ int Fl_Check_Browser::handle(int event) {
 }
 
 //
-// End of "$Id: Fl_Check_Browser.cxx 5985 2007-11-20 21:15:08Z mike $".
+// End of "$Id: Fl_Check_Browser.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
