@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Pack.cxx 6868 2009-09-13 12:10:56Z AlbrechtS $"
+// "$Id: Fl_Pack.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // Packing widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -34,6 +25,16 @@
 #include <FL/Fl_Pack.H>
 #include <FL/fl_draw.H>
 
+/**
+  Creates a new Fl_Pack widget using the given position, size,
+  and label string. The default boxtype is FL_NO_BOX.
+  <P>The destructor <I>also deletes all the children</I>. This allows a
+  whole tree to be deleted at once, without having to keep a pointer to
+  all the children in the user code. A kludge has been done so the 
+  Fl_Pack and all of it's children can be automatic (local)
+  variables, but you must declare the Fl_Pack<I>first</I>, so
+  that it is destroyed last.
+*/
 Fl_Pack::Fl_Pack(int X, int Y, int W, int H,const char *l)
 : Fl_Group(X, Y, W, H, l) {
   resizable(0);
@@ -143,5 +144,5 @@ void Fl_Pack::draw() {
 }
 
 //
-// End of "$Id: Fl_Pack.cxx 6868 2009-09-13 12:10:56Z AlbrechtS $".
+// End of "$Id: Fl_Pack.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
