@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Slider.cxx 5438 2006-09-17 14:58:25Z mike $"
+// "$Id: Fl_Slider.cxx 9637 2012-07-24 04:37:22Z matt $"
 //
 // Slider widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2011 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -27,23 +18,37 @@
 
 #include <FL/Fl.H>
 #include <FL/Fl_Slider.H>
+#include <FL/Fl_Fill_Slider.H>
+#include <FL/Fl_Hor_Slider.H>
+#include <FL/Fl_Hor_Fill_Slider.H>
+#include <FL/Fl_Hor_Nice_Slider.H>
+#include <FL/Fl_Nice_Slider.H>
 #include <FL/fl_draw.H>
 #include <math.h>
 #include "flstring.h"
 
+
 void Fl_Slider::_Fl_Slider() {
   slider_size_ = 0;
   slider_ = 0; // FL_UP_BOX;
 }
 
-Fl_Slider::Fl_Slider(int X, int Y, int W, int H, const char* l)
-: Fl_Valuator(X, Y, W, H, l) {
+/**
+  Creates a new Fl_Slider widget using the given position,
+  size, and label string. The default boxtype is FL_DOWN_BOX.
+*/
+Fl_Slider::Fl_Slider(int X, int Y, int W, int H, const char* L)
+: Fl_Valuator(X, Y, W, H, L) {
   box(FL_DOWN_BOX);
   _Fl_Slider();
 }
 
-Fl_Slider::Fl_Slider(uchar t, int X, int Y, int W, int H, const char* l)
-  : Fl_Valuator(X, Y, W, H, l) {
+/**
+  Creates a new Fl_Slider widget using the given type, position,
+  size, and label string.
+*/
+Fl_Slider::Fl_Slider(uchar t, int X, int Y, int W, int H, const char* L)
+  : Fl_Valuator(X, Y, W, H, L) {
   type(t);
   box(t==FL_HOR_NICE_SLIDER || t==FL_VERT_NICE_SLIDER ?
       FL_FLAT_BOX : FL_DOWN_BOX);
@@ -59,6 +64,10 @@ void Fl_Slider::slider_size(double v) {
   }
 }
 
+/** 
+  Sets the minimum (a) and maximum (b) values for the valuator widget. 
+  if at least one of the values is changed, a partial redraw is asked.
+*/
 void Fl_Slider::bounds(double a, double b) {
   if (minimum() != a || maximum() != b) {
     Fl_Valuator::bounds(a, b); 
@@ -66,16 +75,20 @@ void Fl_Slider::bounds(double a, double b) {
   }
 }
 
-int Fl_Slider::scrollvalue(int p, int W, int t, int l) {
-//	p = position, first line displayed
-//	w = window, number of lines displayed
-//	t = top, number of first line
-//	l = length, total number of lines
+/**
+  Sets the size and position of the sliding knob in the box.
+  \param[in] pos position of first line displayed
+  \param[in] size size of window in lines
+  \param[in] first number of first line
+  \param[in] total total number of lines
+  Returns Fl_Valuator::value(p)
+ */
+int Fl_Slider::scrollvalue(int pos, int size, int first, int total) {
   step(1, 1);
-  if (p+W > t+l) l = p+W-t;
-  slider_size(W >= l ? 1.0 : double(W)/double(l));
-  bounds(t, l-W+t);
-  return value(p);
+  if (pos+size > first+total) total = pos+size-first;
+  slider_size(size >= total ? 1.0 : double(size)/double(total));
+  bounds(first, total-size+first);
+  return value(pos);
 }
 
 // All slider interaction is done as though the slider ranges from
@@ -206,10 +219,14 @@ void Fl_Slider::draw() {
 }
 
 int Fl_Slider::handle(int event, int X, int Y, int W, int H) {
+  // Fl_Widget_Tracker wp(this);
   switch (event) {
-  case FL_PUSH:
+  case FL_PUSH: {
+    Fl_Widget_Tracker wp(this);
     if (!Fl::event_inside(X, Y, W, H)) return 0;
     handle_push();
+    if (wp.deleted()) return 1; }
+    // fall through ...
   case FL_DRAG: {
 
     double val;
@@ -277,34 +294,44 @@ int Fl_Slider::handle(int event, int X, int Y, int W, int H) {
   case FL_RELEASE:
     handle_release();
     return 1;
-  case FL_KEYBOARD :
-    switch (Fl::event_key()) {
-      case FL_Up:
-        if (horizontal()) return 0;
-        handle_push();
-	handle_drag(clamp(increment(value(),-1)));
-	handle_release();
-	return 1;
-      case FL_Down:
-        if (horizontal()) return 0;
-        handle_push();
-	handle_drag(clamp(increment(value(),1)));
-	handle_release();
-	return 1;
-      case FL_Left:
-        if (!horizontal()) return 0;
-        handle_push();
-	handle_drag(clamp(increment(value(),-1)));
-	handle_release();
-	return 1;
-      case FL_Right:
-        if (!horizontal()) return 0;
-        handle_push();
-	handle_drag(clamp(increment(value(),1)));
-	handle_release();
-	return 1;
-      default:
-        return 0;
+  case FL_KEYBOARD:
+    { Fl_Widget_Tracker wp(this);
+      switch (Fl::event_key()) {
+	case FL_Up:
+	  if (horizontal()) return 0;
+	  handle_push();
+	  if (wp.deleted()) return 1;
+	  handle_drag(clamp(increment(value(),-1)));
+	  if (wp.deleted()) return 1;
+	  handle_release();
+	  return 1;
+	case FL_Down:
+	  if (horizontal()) return 0;
+	  handle_push();
+	  if (wp.deleted()) return 1;
+	  handle_drag(clamp(increment(value(),1)));
+	  if (wp.deleted()) return 1;
+	  handle_release();
+	  return 1;
+	case FL_Left:
+	  if (!horizontal()) return 0;
+	  handle_push();
+	  if (wp.deleted()) return 1;
+	  handle_drag(clamp(increment(value(),-1)));
+	  if (wp.deleted()) return 1;
+	  handle_release();
+	  return 1;
+	case FL_Right:
+	  if (!horizontal()) return 0;
+	  handle_push();
+	  if (wp.deleted()) return 1;
+	  handle_drag(clamp(increment(value(),1)));
+	  if (wp.deleted()) return 1;
+	  handle_release();
+	  return 1;
+	default:
+	  return 0;
+      }
     }
     // break not required because of switch...
   case FL_FOCUS :
@@ -334,6 +361,42 @@ int Fl_Slider::handle(int event) {
 		h()-Fl::box_dh(box()));
 }
 
+
+Fl_Fill_Slider::Fl_Fill_Slider(int X,int Y,int W,int H,const char *L)
+: Fl_Slider(X,Y,W,H,L) 
+{
+  type(FL_VERT_FILL_SLIDER);
+}
+
+
+Fl_Hor_Slider::Fl_Hor_Slider(int X,int Y,int W,int H,const char *l)
+: Fl_Slider(X,Y,W,H,l) {
+  type(FL_HOR_SLIDER);
+}
+
+
+Fl_Hor_Fill_Slider::Fl_Hor_Fill_Slider(int X,int Y,int W,int H,const char *L)
+: Fl_Slider(X,Y,W,H,L) 
+{
+  type(FL_HOR_FILL_SLIDER);
+}
+
+
+Fl_Hor_Nice_Slider::Fl_Hor_Nice_Slider(int X,int Y,int W,int H,const char *L)
+: Fl_Slider(X,Y,W,H,L) 
+{
+  type(FL_HOR_NICE_SLIDER); 
+  box(FL_FLAT_BOX);
+}
+
+
+Fl_Nice_Slider::Fl_Nice_Slider(int X,int Y,int W,int H,const char *L)
+: Fl_Slider(X,Y,W,H,L) {
+  type(FL_VERT_NICE_SLIDER); 
+  box(FL_FLAT_BOX);
+}
+
+
 //
-// End of "$Id: Fl_Slider.cxx 5438 2006-09-17 14:58:25Z mike $".
+// End of "$Id: Fl_Slider.cxx 9637 2012-07-24 04:37:22Z matt $".
 //
