@@ -1,24 +1,15 @@
 //
-// "$Id: fl_dnd_x.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id: fl_dnd_x.cxx 9979 2013-09-20 03:36:02Z greg.ercolano $"
 //
 // Drag & Drop code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -43,6 +34,7 @@ extern Atom fl_XdndActionCopy;
 extern Atom fl_XdndFinished;
 //extern Atom fl_XdndProxy;
 extern Atom fl_XdndURIList;
+extern Atom fl_XaUtf8String;
 
 extern char fl_i_own_selection[2];
 extern char *fl_selection_buffer[2];
@@ -63,9 +55,11 @@ static int dnd_aware(Window& window) {
 		     0, 4, False, XA_ATOM,
 		     &actual, &format,
 		     &count, &remaining, &data);
+  int ret = 0;
   if (actual == XA_ATOM && format==32 && count && data)
-    return int(*(Atom*)data);
-  return 0;
+    ret = int(*(Atom*)data);
+  if (data) { XFree(data); data = 0; }
+  return ret;
 }
 
 static int grabfunc(int event) {
@@ -87,7 +81,7 @@ static int local_handle(int event, Fl_Window* window) {
 
 int Fl::dnd() {
   Fl_Window *source_fl_win = Fl::first_window();
-  Fl::first_window()->cursor((Fl_Cursor)21);
+  Fl::first_window()->cursor(FL_CURSOR_MOVE);
   Window source_window = fl_xid(Fl::first_window());
   fl_local_grab = grabfunc;
   Window target_window = 0;
@@ -149,7 +143,7 @@ int Fl::dnd() {
         } else {
 	  // Send plain text...
 	  fl_sendClientMessage(target_window, fl_XdndEnter, source_window,
-			       dndversion<<24, XA_STRING, 0, 0);
+			       dndversion<<24, fl_XaUtf8String, 0, 0);
 	}
       }
     }
@@ -197,5 +191,5 @@ int Fl::dnd() {
 
 
 //
-// End of "$Id: fl_dnd_x.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id: fl_dnd_x.cxx 9979 2013-09-20 03:36:02Z greg.ercolano $".
 //
