@@ -1,24 +1,15 @@
 //
-// "$Id: fl_overlay.cxx 5614 2007-01-18 15:25:09Z matt $"
+// "$Id: fl_overlay.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // Overlay support for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -48,24 +39,22 @@ static int bgx, bgy, bgw, bgh;
 
 static void draw_current_rect() {
 #ifdef USE_XOR
-# ifdef WIN32
+# if defined(USE_X11)
+  XSetFunction(fl_display, fl_gc, GXxor);
+  XSetForeground(fl_display, fl_gc, 0xffffffff);
+  XDrawRectangle(fl_display, fl_window, fl_gc, px, py, pw, ph);
+  XSetFunction(fl_display, fl_gc, GXcopy);
+# elif defined(WIN32)
   int old = SetROP2(fl_gc, R2_NOT);
   fl_rect(px, py, pw, ph);
   SetROP2(fl_gc, old);
-# elif defined(__APPLE_QD__)
-  PenMode( patXor );
-  fl_rect(px, py, pw, ph);
-  PenMode( patCopy );
 # elif defined(__APPLE_QUARTZ__)
   // warning: Quartz does not support xor drawing
   // Use the Fl_Overlay_Window instead.
   fl_color(FL_WHITE);
   fl_rect(px, py, pw, ph);
 # else
-  XSetFunction(fl_display, fl_gc, GXxor);
-  XSetForeground(fl_display, fl_gc, 0xffffffff);
-  XDrawRectangle(fl_display, fl_window, fl_gc, px, py, pw, ph);
-  XSetFunction(fl_display, fl_gc, GXcopy);
+#  error unsupported platform
 # endif
 #else
   if (bgN) { free(bgN); bgN = 0L; }
@@ -105,10 +94,16 @@ static void erase_current_rect() {
 #endif
 }
 
+/**
+  Erase a selection rectangle without drawing a new one
+  */
 void fl_overlay_clear() {
   if (pw > 0) {erase_current_rect(); pw = 0;}
 }
 
+/**
+  Draws a selection rectangle, erasing a previous one by XOR'ing it first.
+  */
 void fl_overlay_rect(int x, int y, int w, int h) {
   if (w < 0) {x += w; w = -w;} else if (!w) w = 1;
   if (h < 0) {y += h; h = -h;} else if (!h) h = 1;
@@ -121,5 +116,5 @@ void fl_overlay_rect(int x, int y, int w, int h) {
 }
 
 //
-// End of "$Id: fl_overlay.cxx 5614 2007-01-18 15:25:09Z matt $".
+// End of "$Id: fl_overlay.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
