@@ -1,24 +1,15 @@
 //
-// "$Id: forms_timer.cxx 6928 2009-11-07 21:11:16Z matt $"
+// "$Id: forms_timer.cxx 10413 2014-10-30 09:16:49Z cand $"
 //
 // Forms timer object for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -32,6 +23,7 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Timer.H>
 #include <FL/fl_draw.H>
+#include <FL/forms.H>
 #ifdef WIN32
 #  ifdef __MWERKS__
 #    include <time.h>
@@ -57,7 +49,7 @@ void fl_gettime(long* sec, long* usec) {
 # else
   struct timeb tp;
   ftime(&tp);
-  *sec = tp.time;
+  *sec = (long) tp.time;
   *usec = tp.millitm * 1000;
 # endif
 #else
@@ -126,23 +118,38 @@ int Fl_Timer::handle(int event) {
   return 0;
 }
 
+/**
+ Destroys the timer and removes the timeout.
+*/
 Fl_Timer::~Fl_Timer() {
   Fl::remove_timeout(stepcb, this);
 }
 
+/**
+  Creates a new Fl_Timer widget using the given type, position,
+  size, and label string. The type parameter can be any of the
+  following symbolic constants:
+
+  \li	FL_NORMAL_TIMER - The timer just does the callback and
+	displays the string "Timer" in the widget.
+  \li	FL_VALUE_TIMER - The timer does the callback and displays
+	the current timer value in the widget.
+  \li	FL_HIDDEN_TIMER - The timer just does the callback and
+	does not display anything.
+*/
 Fl_Timer::Fl_Timer(uchar t, int X, int Y, int W, int H, const char* l)
+
 : Fl_Widget(X, Y, W, H, l) {
   box(FL_DOWN_BOX);
   selection_color(FL_RED);
   delay = 0;
   on = 0;
   direction_ = 0;
-  lastsec = lastusec = total = 0;
   type(t);
   if (t == FL_HIDDEN_TIMER) clear_visible();
   if (t == FL_VALUE_TIMER) align(FL_ALIGN_LEFT);
 }
-
+/** Sets the current timer value */
 void Fl_Timer::value(double d) {
   delay = total = d;
   on = (d > 0.0);
@@ -152,6 +159,7 @@ void Fl_Timer::value(double d) {
   if (on) Fl::add_timeout(FL_TIMER_BLINKRATE, stepcb, this);
 }
 
+/** Gets or sets whether the timer is suspended.*/
 void Fl_Timer::suspended(char d) {
   if (!d) {
     if (on) return;
@@ -166,5 +174,5 @@ void Fl_Timer::suspended(char d) {
 }
 
 //
-// End of "$Id: forms_timer.cxx 6928 2009-11-07 21:11:16Z matt $".
+// End of "$Id: forms_timer.cxx 10413 2014-10-30 09:16:49Z cand $".
 //
