@@ -1,24 +1,15 @@
 //
-// "$Id: glut_compatability.cxx 5845 2007-05-20 00:01:06Z mike $"
+// "$Id: glut_compatability.cxx 9697 2012-10-08 10:32:05Z manolo $"
 //
 // GLUT emulation routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2007 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -49,6 +40,8 @@
 #  define MAXWINDOWS 32
 static Fl_Glut_Window *windows[MAXWINDOWS+1];
 
+static void (*glut_idle_func)() = 0; // global glut idle function
+
 Fl_Glut_Window *glut_window;
 int glut_menu;
 void (*glut_menustate_function)(int);
@@ -103,8 +96,8 @@ int Fl_Glut_Window::handle(int event) {
 
   case FL_MOUSEWHEEL:
     button = Fl::event_dy();
-    while (button < 0) {mouse(3,GLUT_DOWN,ex,ey); ++button;}
-    while (button > 0) {mouse(4,GLUT_DOWN,ex,ey); --button;}
+    while (button < 0) {if (mouse) mouse(3,GLUT_DOWN,ex,ey); ++button;}
+    while (button > 0) {if (mouse) mouse(4,GLUT_DOWN,ex,ey); --button;}
     return 1;
 
   case FL_RELEASE:
@@ -185,9 +178,11 @@ void Fl_Glut_Window::_init() {
   mode(glut_mode);
 }
 
+/** Creates a glut window, registers to the glut windows list.*/
 Fl_Glut_Window::Fl_Glut_Window(int W, int H, const char *t) :
   Fl_Gl_Window(W,H,t) {_init();}
 
+/** Creates a glut window, registers to the glut windows list.*/
 Fl_Glut_Window::Fl_Glut_Window(int X,int Y,int W,int H, const char *t) :
   Fl_Gl_Window(X,Y,W,H,t) {_init();}
 
@@ -215,7 +210,7 @@ void glutMainLoop() {Fl::run();}
 
 ////////////////////////////////////////////////////////////////
 
-static int initx, inity, initw=300, inith=300, initpos;
+static int initx=0, inity=0, initw=300, inith=300, initpos=0;
 
 void glutInitWindowPosition(int x, int y) {
   initx = x; inity = y; initpos = 1;
@@ -257,7 +252,7 @@ int glutCreateSubWindow(int win, int x, int y, int w, int h) {
   W->make_current();
   return W->number;
 }
-
+/** Destroys the glut window, first unregister it from the glut windows list */
 Fl_Glut_Window::~Fl_Glut_Window() {
   if (glut_window == this) glut_window = 0;
   windows[number] = 0;
@@ -338,7 +333,7 @@ void glutAddMenuEntry(char *label, int value) {
   Fl_Menu_Item* i = additem(m);
   i->text = label;
   i->callback_ = (Fl_Callback*)(m->cb);
-  i->user_data_ = (void *)value;
+  i->argument(value);
 }
 
 void glutAddSubMenu(char *label, int submenu) {
@@ -355,7 +350,7 @@ void glutChangeToMenuEntry(int item, char *label, int value) {
   Fl_Menu_Item* i = &m->m[item-1];
   i->text = label;
   i->callback_ = (Fl_Callback*)(m->cb);
-  i->user_data_ = (void *)value;
+  i->argument(value);
   i->flags = 0;
 }
 
@@ -497,8 +492,22 @@ int glutExtensionSupported( const char* extension )
   }
 }
 
+// Add a mechanism to handle adding/removing the glut idle function
+// without depending on the (deprecated) set_idle method.
+void glutIdleFunc(void (*f)())
+{
+  // no change
+  if(glut_idle_func == f) return;
+  // remove current idle
+  if(glut_idle_func) Fl::remove_idle((void (*)(void *))glut_idle_func);
+  // install new idle func - if one was passed
+  if(f) Fl::add_idle((void (*)(void *))f);
+  // record new idle func - even if it is NULL
+  glut_idle_func = f;
+} // glutIdleFunc
+
 #endif // HAVE_GL
 
 //
-// End of "$Id: glut_compatability.cxx 5845 2007-05-20 00:01:06Z mike $".
+// End of "$Id: glut_compatability.cxx 9697 2012-10-08 10:32:05Z manolo $".
 //
