@@ -1,24 +1,15 @@
 //
-// "$Id: curve.cxx 7978 2010-12-08 14:00:35Z AlbrechtS $"
+// "$Id: curve.cxx 9392 2012-04-24 02:06:52Z fabien $"
 //
 // Curve test program for the Fast Light Tool Kit (FLTK).
 //
 // Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -100,7 +91,11 @@ int main(int argc, char** argv) {
     s->step(1);
     s->value(args[n]);
     s->align(FL_ALIGN_LEFT);
+#ifdef __LP64__
+    s->callback(slider_cb, (void*) (long long)n);
+#else
     s->callback(slider_cb, (void*)n);
+#endif
   }
   Fl_Toggle_Button but(50,y,50,25,"points");
   but.callback(points_cb);
@@ -111,5 +106,5 @@ int main(int argc, char** argv) {
 }
 
 //
-// End of "$Id: curve.cxx 7978 2010-12-08 14:00:35Z AlbrechtS $".
+// End of "$Id: curve.cxx 9392 2012-04-24 02:06:52Z fabien $".
 //
